;; -*- Lisp -*-
;; $Id: $ 
;; shinyblue/theme.jl

;; Copyright (C) 1999 SATO Satoru <satoru@d1.dion.ne.jp>

;; This theme is based on the Mike Bond's Absolute Sawmill theme
;; and modified by SATO Satoru <satoru@d1.dion.ne.jp>. All image
;; components were taken from the ShinyBlue enlightenment theme by
;; Paul Schultz <khyron@choice.net>.

;; preferences

(defgroup shinyblue-frame "ShinyBlue")

(defcustom shinyblue:foreground "white"
  "Title foreground color"
  :group shinyblue-frame
  :type color)

(defcustom shinyblue:full-title t
  "Use full width of window for title and buttons"
  :group shinyblue-frame
  :type boolean)

(defcustom shinyblue:shrink-shaped-title nil
  "Shrink title to width of title text when shaded"
  :group shinyblue-frame
  :type boolean)

(defcustom shinyblue:border-dimension 4
  "Width of window border"
  :group shinyblue-frame
  :type number
  :range (1 . nil))

;; frame class: shade-button
(def-frame-class shade-button '((cursor . arrow))
  (bind-keys shade-button-keymap "Button1-Off" 'toggle-window-shaded))

(let*
    ((title-width (lambda (w)
		    (let
			((w-width (car (window-dimensions w))))
		      (cond ((eq shinyblue:full-title 'nil)
			     (max 0 (min (- w-width 80) (max (- (/ w-width 2) 80) (+ 10 (text-width (window-name w)))))))
			    (t
			     (max 0 (- w-width 80)))))))

     (title-width-shaped (lambda (w)
			   (let
			       ((w-width (car (window-dimensions w))))
			     (cond ((eq shinyblue:shrink-shaped-title 'nil)
				    (title-width w))
				   (t
				    (max 0 (min (- w-width 80) (+ 10 (text-width (window-name w))))))))))

     (border-dimension (lambda () shinyblue:border-dimension))
     (-border-dimension (lambda () (- shinyblue:border-dimension)))
     (foreground () shinyblue:foreground)

     ;; called after the WM_NAME property changes
     (update-text-width
      (lambda (w)
	(if (eq (window-get w 'current-frame-style) 'shinyblue)
	    (rebuild-frame w))))

     (rebuild
      (lambda()
	(mapc (lambda (w)
		(when (eq (window-get w 'current-frame-style) 'shinyblue)
		  (rebuild-frame w))) (managed-windows))))

     (iconify-images (mapcar (lambda (i)
			       (set-image-border i 4 4 4 4))
			     (list (make-image "iconify_normal.png")
				   (make-image "iconify_normal_active.png")
				   (make-image "iconify_hilited_active.png")
				   (make-image "iconify_clicked_active.png"))))

     (shade-images (mapcar (lambda (i)
			       (set-image-border i 4 4 4 4))
			     (list (make-image "shade_normal.png")
				   (make-image "shade_normal_active.png")
				   (make-image "shade_hilited_active.png")
				   (make-image "shade_clicked_active.png"))))

     (bar-images (mapcar (lambda (i)
			   (set-image-border i 4 4 4 4))
			 (list (make-image "bar_normal.png")
			       (make-image "bar_normal_active.png")
			       (make-image "bar_hilited_active.png")
			       (make-image "bar_clicked_active.png"))))

     (close-images (mapcar (lambda (i)
			     (set-image-border i 4 4 4 4))
			   (list (make-image "close_normal.png")
				 (make-image "close_normal_active.png")
				 (make-image "close_hilited_active.png")
				 (make-image "close_clicked_active.png"))))

     (maximize-images (mapcar (lambda (i)
				(set-image-border i 4 4 4 4))
			      (list (make-image "maximize_normal.png")
				    (make-image "maximize_normal_active.png")
				    (make-image "maximize_hilited_active.png")
				    (make-image "maximize_clicked_active.png"))))

     (menu-images (mapcar (lambda (i)
				(set-image-border i 4 4 4 4))
			      (list (make-image "menu_normal.png")
				    (make-image "menu_normal_active.png")
				    (make-image "menu_hilited_active.png")
				    (make-image "menu_clicked_active.png"))))

     (frame
      `(
	;; iconify button
	((background . ,iconify-images)
	 (left-edge . 0)
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . iconify-button))
	;; shade button
	((background . ,shade-images)
	 (left-edge . 16)
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . shade-button))
	;; title bar
	((background . ,bar-images)
	 (foreground . ,foreground)
	 (text . ,window-name)
	 (x-justify . center)
	 (y-justify . center)
	 (left-edge . 32)
	 (width . ,title-width)
	 (top-edge . -16)
	 (class . title))
	;; maximize button
	((background . ,maximize-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 32)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . maximize-button))
	;; close button
	((background . ,close-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 48)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . close-button))
    ;; menu button
	((background . ,menu-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 64)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . menu-button))
	;; rhs bit of title
	((background . ,bar-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 80)))
	 (right-edge . 0)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-border))
	;; left frame
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . right-border))
	;; right frame
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . left-border))
	;; bottom frame
	((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-border))
	;; top-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-left-corner))
	;; top-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-right-corner))
	;; bottom-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-left-corner))
	;; bottom-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-right-corner))))

     (shaped-frame
      `(
	;; iconify button
	((background . ,iconify-images)
	 (left-edge . 0)
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . iconify-button))
	;; shade button
	((background . ,shade-images)
	 (left-edge . 16)
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . shade-button))
	;; title bar
	((background . ,bar-images)
	 (foreground . ,foreground)
	 (text . ,window-name)
	 (x-justify . center)
	 (y-justify . center)
	 (left-edge . 32)
	 (width . ,title-width-shaped)
	 (top-edge . -16)
	 (class . title))
	;; maximize button
	((background . ,maximize-images)
	 (left-edge . ,(lambda (w) (+ (title-width-shaped w) 32)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . maximize-button))
	;; close button
	((background . ,close-images)
	 (left-edge . ,(lambda (w) (+ (title-width-shaped w) 48)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . close-button))
    ;; menu button
	((background . ,menu-images)
	 (left-edge . ,(lambda (w) (+ (title-width-shaped w) 64)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . menu-button))))

     (transient-frame
      `(((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-border))
	((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-border))
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . left-border))
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . right-border))
	;; top-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-right-corner))
	;; top-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-left-corner))
	;; bottom-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-left-corner))
	;; bottom-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-right-corner))))

     (shaped-transient-frame
      `(((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-border)))))

  (add-frame-style 'shinyblue
		   (lambda (w type)
		     (cond ((eq type 'shaped)
			    shaped-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaped-transient)
			    shaped-transient-frame)
			   ((eq type 'unframed)
			    nil-frame)
			   (t
			    frame))))

  (mapc (lambda (sym)
	  (custom-set-property sym ':after-set rebuild))
	'(shinyblue:foreground
	  shinyblue:full-title
	  shinyblue:shrink-shaped-title
	  shinyblue:border-dimension))

  (call-after-property-changed 'WM_NAME update-text-width))
